(* GNU-GPL
 * System Compression - Logiciel de Compression/Decompression
 * Copyright (c) 2002 Ulrich Van Den Hekke (ulrich.vdh@free.fr)
 * 24 rue Jean Jaures
 * 59560 Comines
 *
 * Ce programme est libre, vous pouvez le redistribuer et/ou le modifier selon
 * les termes de la Licence Publique Gnrale GNU publie par la
 * Free Software Foundation (version 2 ou bien toute autre version ultrieure
 * choisie par vous).
 *
 * Ce programme est distribu car potentiellement utile, mais
 * SANS AUCUNE GARANTIE, ni explicite ni implicite, y compris les garanties de
 * commercialisation ou d'adaptation dans un but spcifique. Reportez-vous  la
 * Licence Publique Gnrale GNU pour plus de dtails.
 *
 * Vous devez avoir reu une copie de la Licence Publique Gnrale GNU en mme
 * temps que ce programme ; si ce n'est pas le cas, crivez  la
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, tats-Unis.
 *
 *)

unit scProgressForm;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, Menus;

type
  TActionProc   = procedure() of object;
  TProgressForm = class(TForm)
    PanelText: TPanel;
    TimerOperation: TTimer;
    procedure TimerOperationTimer(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
  private
    FActionProc: TActionProc;
    procedure SetActionProc(const Value: TActionProc);
    { Dclarations prives }
  public
    { Dclarations publiques }
    property ActionProc: TActionProc read FActionProc write SetActionProc;
  end;                                       

var
  ProgressForm: TProgressForm;

procedure CreateProgressDialogForm(const InitText: string; const ActionProc: TActionProc; const Show: boolean = true);

implementation

{$R *.DFM}

procedure CreateProgressDialogForm(const InitText: string; const ActionProc: TActionProc; const Show: boolean = true);
begin
  ProgressForm := TProgressForm.Create(Application.MainForm);
  try
    ProgressForm.PanelText.Caption      := InitText;
    ProgressForm.ActionProc             := ActionProc;
    ProgressForm.TimerOperation.Enabled := true;
    if(Show) then ProgressForm.ShowModal;
    if(not Show) then while(Assigned(ProgressForm.FActionProc)) do Application.ProcessMessages;
  finally
    ProgressForm.Free;
  end;
end;

{ TProgressForm }

procedure TProgressForm.SetActionProc(const Value: TActionProc);
begin
  FActionProc := Value;
end;

procedure TProgressForm.TimerOperationTimer(Sender: TObject);
begin
  TimerOperation.Enabled := false;
  if(Assigned(FActionProc)) then FActionProc();
  FActionProc := nil;
  if(Visible) then Close();
end;

procedure TProgressForm.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  if(Assigned(FActionProc)) then
    Action := caNone;
end;

end.
