(* GNU-GPL
 * System Compression - Logiciel de Compression/Decompression
 * Copyright (c) 2002 Ulrich Van Den Hekke (ulrich.vdh@free.fr)
 * 24 rue Jean Jaures
 * 59560 Comines
 *
 * Ce programme est libre, vous pouvez le redistribuer et/ou le modifier selon
 * les termes de la Licence Publique Gnrale GNU publie par la
 * Free Software Foundation (version 2 ou bien toute autre version ultrieure
 * choisie par vous).
 *
 * Ce programme est distribu car potentiellement utile, mais
 * SANS AUCUNE GARANTIE, ni explicite ni implicite, y compris les garanties de
 * commercialisation ou d'adaptation dans un but spcifique. Reportez-vous  la
 * Licence Publique Gnrale GNU pour plus de dtails.
 *
 * Vous devez avoir reu une copie de la Licence Publique Gnrale GNU en mme
 * temps que ce programme ; si ce n'est pas le cas, crivez  la
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, tats-Unis.
 *
 *)

(******************************************************************************)
(* Utilitaire de compression en mode console                                  *)
(* Version : 1.0.0.0                                                          *)
(* Author : Ulrich Van Den Hekke                                              *)
(* Create day : 10/07/2002                                                    *)
(* -------------------------------------------------------------------------- *)
(* Serial : 54E73B-02071036-077B446A                                          *)
(* Object : Un utilitaire de compression en ligne de commande                 *)
(******************************************************************************)

program syscons;

uses
  SysUtils,
  Classes,
  VerManag,
  ConsoleManager,
  FileTools,
  Common in 'Common.pas',
  AlgoList in 'AlgoList.pas',
  scFileStruct in 'scFileStruct.pas',
  sysconsconst in 'sysconsconst.pas',
  sysconsutils in 'sysconsutils.pas',
  syscconst in 'syscconst.pas',
  syscutils in 'syscutils.pas',
  reginfos in 'reginfos.pas';

{$R *.RES}

var
  PlgList        : TAlgoPluginsList;
  PosParam       : Integer;

  ArchiveName    : TFileName;
  Archive        : TSystemCFileStruct;
  CurrentFile    : TFileStream;

  IsRecurse      : boolean;
  CurrAlgoFile   : string;
  CurrFile       : string;
  CurrFileIndex  : integer;
  CurrDestDir    : string;
  Pause          : boolean = false;

  LastProgressPos: Byte;

function showProgress(const Algo: string; const Position, Max: LongWord): boolean;
var Percent: Byte;
begin
  Percent := Round((Position / Max) * 100);
  if(Percent <> LastProgressPos) then
  begin
    Write(Format(rsProgress, [Trim(Algo), Percent, SizeToStr(Position, bytesuntis)]));
    LastProgressPos := Percent;
  end;
  result := true;
end;

function OnAskNewDisk(const DiskNum: Byte; var Stream: TStream; var MaxSize: LongInt): boolean;
begin
  result := true;

  CurrentFile.Free;
  while(DiskFree(ArchiveName) <= cstMinSize) do begin
    Write(Format(rsInsertNewDsk, [UpCase(ArchiveName[1])]));
    ReadLn;
  end;
  ArchiveName := GenerateFileName(ArchiveName, DiskNum);
  CurrentFile := TFileStream.Create(ArchiveName, fmCreate or fmShareDenyWrite);
  MaxSize     := CalcFileSize(CurrentFile, ArchiveName);
  Stream      := CurrentFile;
end;

function OnAskOneDisk(const DiskNum: Byte; var Stream: TStream; var MaxSize: LongInt): boolean;
begin
  result := true;

  CurrentFile.Free;
  ArchiveName := GenerateFileName(ArchiveName, DiskNum);
  while(not FileExists(ArchiveName)) do begin
    Write(Format(rsInsertOneDsk, [DiskNum, ArchiveName[1]]));
    ReadLn;
  end;
  CurrentFile := TFileStream.Create(ArchiveName, fmOpenReadWrite or fmShareDenyWrite);
  MaxSize := CalcFileSize(CurrentFile, ArchiveName);
  Stream := CurrentFile;
end;

begin
  WriteLn(HeadGenerator() + #10);
  PlgList := TAlgoPluginsList.Create(RegInfosOption.PluginsDir);
  try
    PlgList.OnProgression := showProgress;
    // ParamCount <> 0 ?
    if(ParamCount = 0) then showHelp();
    // Test des options
    PosParam := 1;
    while(PosParam <= ParamCount) do begin
      if(ParamStr(PosParam) = '--help') then showHelp() else
      if(ParamStr(PosParam) = '--credits') then showCredits() else
      if(ParamStr(PosParam) = '--licence') then showLicence() else
      if((ParamStr(PosParam) = '--pause') or (ParamStr(PosParam) = '-p')) then Pause := not Pause else
      if(ParamStr(PosParam) = '-g') then begin
        Inc(PosParam);
        TestAlgo(PlgList, ParamStr(PosParam), showProgress);
      end else
      if(ParamStr(PosParam) = '-l') then begin
        Inc(PosParam);
        if(ParamStr(PosParam) <> EmptyStr) then begin
          ArchiveName := ParamStr(PosParam);
          Inc(PosParam);
          Archive     := nil;
          try
            InitialiseArchive(ArchiveName, CurrentFile);
            Archive := TSystemCFileStruct.Create(CurrentFile, LongWord(CalcFileSize(CurrentFile, ArchiveName)),
                                                 PlgList, OnAskNewDisk, OnAskOneDisk);
            showFileList(Archive, ParamStr(PosParam));
          finally
            Archive.Free;
            FinalliseArchive(CurrentFile);
          end;
        end else
          showPluginsList(PlgList);
      end else
      if(ParamStr(PosParam) = '-a') then begin
        Inc(PosParam);
        ArchiveName := ParamStr(PosParam);
        Inc(PosParam);
        Archive     := nil;
        try
          InitialiseArchive(ArchiveName, CurrentFile);
          Archive := TSystemCFileStruct.Create(CurrentFile, CalcFileSize(CurrentFile, ArchiveName),
                                                 PlgList, OnAskNewDisk, OnAskOneDisk);
          Archive.OnProgress := showProgress;
          while(PosParam <= ParamCount) do begin
            IsRecurse    := false;
            CurrAlgoFile := cstAutoPlugin;
            while((PosParam <= ParamCount) and (ParamStr(PosParam)[1] = '-')) do begin
              if(ParamStr(PosParam) = '-r') then IsRecurse := not IsRecurse else
              if(ParamStr(PosParam) = '-g') then begin
                Inc(PosParam);
                CurrAlgoFile := ParamStr(PosParam);
              end else
                showHelp();
              Inc(PosParam);
            end;
            CurrFile := ExpandFileName(ParamStr(PosParam));

            AddFiles(Archive, ExtractFilePath(CurrFile), ExtractFilePath(CurrFile), ExtractFileName(CurrFile), CurrAlgoFile, IsRecurse);
            Inc(PosParam);
          end;
        finally
          Archive.Free;
          FinalliseArchive(CurrentFile);
        end;
      end else
      if(ParamStr(PosParam) = '-e') then begin
        Inc(PosParam);
        ArchiveName := ParamStr(PosParam);
        Inc(PosParam);
        Archive     := nil;
        try
          InitialiseArchive(ArchiveName, CurrentFile);
          Archive := TSystemCFileStruct.Create(CurrentFile, LongWord(CalcFileSize(CurrentFile, ArchiveName)),
                                                 PlgList, OnAskNewDisk, OnAskOneDisk);
          Archive.OnProgress := showProgress;
          CurrDestDir := NormalDir(GetCurrentDir);
          if(PosParam > ParamCount) then ExtractFile(Archive, CurrDestDir, '', 1);
          while(PosParam <= ParamCount) do begin
            CurrFileIndex := 1;
            while((PosParam <= ParamCount) and (ParamStr(PosParam)[1] = '-')) do begin
              if(ParamStr(PosParam) = '-d') then begin
                Inc(PosParam);
                CurrDestDir := ExpandFileName(NormalDir(ParamStr(PosParam)));
              end else
                showHelp();
              Inc(PosParam);
            end;
            CurrFile := ParamStr(PosParam);
            if(pos(':', CurrFile) <> 0) then begin
              CurrFileIndex := StrToInt(Copy(CurrFile, Pos(':', CurrFile) + 1, length(CurrFile)));
              CurrFile      := Copy(CurrFile, 1, pos(':', CurrFile) - 1);
            end;

            ExtractFile(Archive, CurrDestDir, CurrFile, CurrFileIndex);

            Inc(PosParam);
          end;
      finally
          Archive.Free;
          FinalliseArchive(CurrentFile);
        end;
      end else
      if(ParamStr(PosParam) = '-d') then begin
        Inc(PosParam);
        ArchiveName := ParamStr(PosParam);
        Inc(PosParam);
        Archive     := nil;
        try
          InitialiseArchive(ArchiveName, CurrentFile);
          Archive := TSystemCFileStruct.Create(CurrentFile, LongWord(CalcFileSize(CurrentFile, ArchiveName)),
                                                 PlgList, OnAskNewDisk, OnAskOneDisk);
          Archive.OnProgress := showProgress;
          if(PosParam > ParamCount) then begin
            DeleteFile(Archive);
            FinalliseArchive(CurrentFile);
            SysUtils.DeleteFile(ArchiveName);
            WriteLn(Format(rsDeleteFinish, [ExtractFileName(ArchiveName)]));
          end;

          while(PosParam <= ParamCount) do begin
            CurrFileIndex := 1;
            CurrFile      := ParamStr(PosParam);
            if(pos(':', CurrFile) <> 0) then begin
              CurrFileIndex := StrToInt(Copy(CurrFile, Pos(':', CurrFile) + 1, length(CurrFile)));
              CurrFile      := Copy(CurrFile, 1, pos(':', CurrFile) - 1);
            end;

            DeleteFile(Archive, CurrFile, CurrFileIndex);

            Inc(PosParam);
          end;
      finally
          Archive.Free;
          FinalliseArchive(CurrentFile);
        end;
      end else
        showHelp();
      Inc(PosParam);
    end;
  except
    on E: EAbort do ;
    on E: Exception do WriteLn(E.Message);
  end;
  PlgList.Free;
  if(Pause) then begin
    Write(rsPauseApp);
    ReadLn;
  end;
end.
